@@ -1,4 +1,3 @@
-# -*- encoding:utf-8 -*-
 from collections import namedtuple
 from copy import deepcopy
 import re
@@ -9,19 +8,19 @@ import jinja2
 
 from numpydoc.numpydoc import update_config
 from numpydoc.xref import DEFAULT_LINKS
-from numpydoc.docscrape import (
-    NumpyDocString,
-    FunctionDoc,
-    ClassDoc,
-    ParseError
+from numpydoc.docscrape import NumpyDocString, FunctionDoc, ClassDoc, ParseError
+from numpydoc.docscrape_sphinx import (
+    SphinxDocString,
+    SphinxClassDoc,
+    SphinxFunctionDoc,
+    get_doc_object,
 )
-from numpydoc.docscrape_sphinx import (SphinxDocString, SphinxClassDoc,
-                                       SphinxFunctionDoc, get_doc_object)
+import pytest
 from pytest import raises as assert_raises
 from pytest import warns as assert_warns
 
 
-doc_txt = '''\
+doc_txt = """\
   numpy.multivariate_normal(mean, cov, shape=None, spam=None)
 
   Draw values from a multivariate normal distribution with specified
@@ -45,6 +44,8 @@ doc_txt = '''\
       Given a shape of, for example, (m,n,k), m*n*k samples are
       generated, and packed in an m-by-n-by-k arrangement.  Because
       each sample is N-dimensional, the output shape is (m,n,k,N).
+  dtype : data type object, optional (default : float)
+      The type and size of the data to be returned.
 
   Returns
   -------
@@ -110,6 +111,7 @@ doc_txt = '''\
   --------
   some, other, funcs
   otherfunc : relationship
+  :py:meth:`spyder.widgets.mixins.GetHelpMixin.show_object_info`
 
   Examples
   --------
@@ -128,8 +130,13 @@ doc_txt = '''\
   .. index:: random
      :refguide: random;distributions, random;gauss
 
-  '''
-doc = NumpyDocString(doc_txt)
+  """
+
+
+@pytest.fixture(params=["", "\n    "], ids=["flush", "newline_indented"])
+def doc(request):
+    return NumpyDocString(request.param + doc_txt)
+
 
 doc_yields_txt = """
 Test generator
@@ -165,81 +172,93 @@ c : int
 doc_sent = NumpyDocString(doc_sent_txt)
 
 
-def test_signature():
-    assert doc['Signature'].startswith('numpy.multivariate_normal(')
-    assert doc['Signature'].endswith('spam=None)')
+def test_signature(doc):
+    assert doc["Signature"].startswith("numpy.multivariate_normal(")
+    assert doc["Signature"].endswith("spam=None)")
 
 
-def test_summary():
-    assert doc['Summary'][0].startswith('Draw values')
-    assert doc['Summary'][-1].endswith('covariance.')
+def test_summary(doc):
+    assert doc["Summary"][0].startswith("Draw values")
+    assert doc["Summary"][-1].endswith("covariance.")
 
 
-def test_extended_summary():
-    assert doc['Extended Summary'][0].startswith('The multivariate normal')
+def test_extended_summary(doc):
+    assert doc["Extended Summary"][0].startswith("The multivariate normal")
 
 
-def test_parameters():
-    assert len(doc['Parameters']) == 3
-    names = [n for n, _, _ in doc['Parameters']]
-    assert all(a == b for a, b in zip(names, ['mean', 'cov', 'shape']))
+def test_parameters(doc):
+    assert len(doc["Parameters"]) == 4
+    names = [n for n, _, _ in doc["Parameters"]]
+    assert all(a == b for a, b in zip(names, ["mean", "cov", "shape"]))
 
-    arg, arg_type, desc = doc['Parameters'][1]
-    assert arg_type == '(N, N) ndarray'
-    assert desc[0].startswith('Covariance matrix')
-    assert doc['Parameters'][0][-1][-1] == '   (1+2+3)/3'
+    arg, arg_type, desc = doc["Parameters"][1]
+    assert arg_type == "(N, N) ndarray"
+    assert desc[0].startswith("Covariance matrix")
+    assert doc["Parameters"][0][-1][-1] == "   (1+2+3)/3"
+
+    arg, arg_type, desc = doc["Parameters"][2]
+    assert arg == "shape"
+    assert arg_type == "tuple of ints"
+    assert desc[0].startswith("Given")
+    assert doc["Parameters"][0][-1][-1] == "   (1+2+3)/3"
+
+    arg, arg_type, desc = doc["Parameters"][3]
+    assert arg == "dtype"
+    assert arg_type == "data type object, optional (default : float)"
+    assert desc[0].startswith("The type and size")
 
 
-def test_other_parameters():
-    assert len(doc['Other Parameters']) == 1
-    assert [n for n, _, _ in doc['Other Parameters']] == ['spam']
-    arg, arg_type, desc = doc['Other Parameters'][0]
-    assert arg_type == 'parrot'
-    assert desc[0].startswith('A parrot off its mortal coil')
+def test_other_parameters(doc):
+    assert len(doc["Other Parameters"]) == 1
+    assert [n for n, _, _ in doc["Other Parameters"]] == ["spam"]
+    arg, arg_type, desc = doc["Other Parameters"][0]
+    assert arg_type == "parrot"
+    assert desc[0].startswith("A parrot off its mortal coil")
 
 
-def test_returns():
-    assert len(doc['Returns']) == 3
-    arg, arg_type, desc = doc['Returns'][0]
-    assert arg == 'out'
-    assert arg_type == 'ndarray'
-    assert desc[0].startswith('The drawn samples')
-    assert desc[-1].endswith('distribution.')
+def test_returns(doc):
+    assert len(doc["Returns"]) == 3
+    arg, arg_type, desc = doc["Returns"][0]
+    assert arg == "out"
+    assert arg_type == "ndarray"
+    assert desc[0].startswith("The drawn samples")
+    assert desc[-1].endswith("distribution.")
 
-    arg, arg_type, desc = doc['Returns'][1]
-    assert arg == ''
-    assert arg_type == 'list of str'
-    assert desc[0].startswith('This is not a real')
-    assert desc[-1].endswith('anonymous return values.')
+    arg, arg_type, desc = doc["Returns"][1]
+    assert arg == ""
+    assert arg_type == "list of str"
+    assert desc[0].startswith("This is not a real")
+    assert desc[-1].endswith("anonymous return values.")
 
-    arg, arg_type, desc = doc['Returns'][2]
-    assert arg == ''
-    assert arg_type == 'no_description'
-    assert not ''.join(desc).strip()
+    arg, arg_type, desc = doc["Returns"][2]
+    assert arg == ""
+    assert arg_type == "no_description"
+    assert not "".join(desc).strip()
 
 
 def test_yields():
-    section = doc_yields['Yields']
+    section = doc_yields["Yields"]
     assert len(section) == 3
-    truth = [('a', 'int', 'apples.'),
-             ('b', 'int', 'bananas.'),
-             ('', 'int', 'unknowns.')]
+    truth = [
+        ("a", "int", "apples."),
+        ("b", "int", "bananas."),
+        ("", "int", "unknowns."),
+    ]
     for (arg, arg_type, desc), (arg_, arg_type_, end) in zip(section, truth):
         assert arg == arg_
         assert arg_type == arg_type_
-        assert desc[0].startswith('The number of')
+        assert desc[0].startswith("The number of")
         assert desc[0].endswith(end)
 
 
 def test_sent():
-    section = doc_sent['Receives']
+    section = doc_sent["Receives"]
     assert len(section) == 2
-    truth = [('b', 'int', 'bananas.'),
-             ('c', 'int', 'oranges.')]
+    truth = [("b", "int", "bananas."), ("c", "int", "oranges.")]
     for (arg, arg_type, desc), (arg_, arg_type_, end) in zip(section, truth):
         assert arg == arg_
         assert arg_type == arg_type_
-        assert desc[0].startswith('The number of')
+        assert desc[0].startswith("The number of")
         assert desc[0].endswith(end)
 
 
@@ -275,7 +294,8 @@ Notes
 -----
 That should break...
 """
-    assert_raises(ValueError, NumpyDocString, doc_text)
+    with pytest.raises(ValueError, match="The section Notes appears twice"):
+        NumpyDocString(doc_text)
 
     # if we have a numpydoc object, we know where the error came from
     class Dummy:
@@ -291,6 +311,7 @@ That should break...
         Second note.
 
         """
+
         def spam(self, a, b):
             """Spam\n\nSpam spam."""
             pass
@@ -312,67 +333,60 @@ That should break...
         Second note.
         """
 
-    try:
+    with pytest.raises(ValueError, match="Dummy class"):
         SphinxClassDoc(Dummy)
-    except ValueError as e:
-        # python 3 version or python 2 version
-        assert ("test_section_twice.<locals>.Dummy" in str(e)
-                or 'test_docscrape.Dummy' in str(e))
 
-    try:
+    with pytest.raises(ValueError, match="dummy_func"):
         SphinxFunctionDoc(dummy_func)
-    except ValueError as e:
-        # python 3 version or python 2 version
-        assert ("test_section_twice.<locals>.dummy_func" in str(e)
-                or 'function dummy_func' in str(e))
 
 
-def test_notes():
-    assert doc['Notes'][0].startswith('Instead')
-    assert doc['Notes'][-1].endswith('definite.')
-    assert len(doc['Notes']) == 17
+def test_notes(doc):
+    assert doc["Notes"][0].startswith("Instead")
+    assert doc["Notes"][-1].endswith("definite.")
+    assert len(doc["Notes"]) == 17
 
 
-def test_references():
-    assert doc['References'][0].startswith('..')
-    assert doc['References'][-1].endswith('2001.')
+def test_references(doc):
+    assert doc["References"][0].startswith("..")
+    assert doc["References"][-1].endswith("2001.")
 
 
-def test_examples():
-    assert doc['Examples'][0].startswith('>>>')
-    assert doc['Examples'][-1].endswith('True]')
+def test_examples(doc):
+    assert doc["Examples"][0].startswith(">>>")
+    assert doc["Examples"][-1].endswith("True]")
 
 
-def test_index():
-    assert doc['index']['default'] == 'random'
-    assert len(doc['index']) == 2
-    assert len(doc['index']['refguide']) == 2
+def test_index(doc):
+    assert doc["index"]["default"] == "random"
+    assert len(doc["index"]) == 2
+    assert len(doc["index"]["refguide"]) == 2
 
 
 def _strip_blank_lines(s):
     "Remove leading, trailing and multiple blank lines"
-    s = re.sub(r'^\s*\n', '', s)
-    s = re.sub(r'\n\s*$', '', s)
-    s = re.sub(r'\n\s*\n', r'\n\n', s)
+    s = re.sub(r"^\s*\n", "", s)
+    s = re.sub(r"\n\s*$", "", s)
+    s = re.sub(r"\n\s*\n", r"\n\n", s)
     return s
 
 
 def line_by_line_compare(a, b, n_lines=None):
     a = textwrap.dedent(a)
     b = textwrap.dedent(b)
-    a = [l.rstrip() for l in _strip_blank_lines(a).split('\n')][:n_lines]
-    b = [l.rstrip() for l in _strip_blank_lines(b).split('\n')][:n_lines]
+    a = [l.rstrip() for l in _strip_blank_lines(a).split("\n")][:n_lines]
+    b = [l.rstrip() for l in _strip_blank_lines(b).split("\n")][:n_lines]
     assert len(a) == len(b)
     for ii, (aa, bb) in enumerate(zip(a, b)):
         assert aa == bb
 
 
-def test_str():
+def test_str(doc):
     # doc_txt has the order of Notes and See Also sections flipped.
     # This should be handled automatically, and so, one thing this test does
     # is to make sure that See Also precedes Notes in the output.
-    line_by_line_compare(str(doc),
-"""numpy.multivariate_normal(mean, cov, shape=None, spam=None)
+    line_by_line_compare(
+        str(doc),
+        """numpy.multivariate_normal(mean, cov, shape=None, spam=None)
 
 Draw values from a multivariate normal distribution with specified
 mean and covariance.
@@ -394,6 +408,8 @@ shape : tuple of ints
     Given a shape of, for example, (m,n,k), m*n*k samples are
     generated, and packed in an m-by-n-by-k arrangement.  Because
     each sample is N-dimensional, the output shape is (m,n,k,N).
+dtype : data type object, optional (default : float)
+    The type and size of the data to be returned.
 
 Returns
 -------
@@ -435,6 +451,8 @@ See Also
     ..
 `otherfunc`_
     relationship
+:py:meth:`spyder.widgets.mixins.GetHelpMixin.show_object_info`
+    ..
 
 Notes
 -----
@@ -478,12 +496,14 @@ standard deviation:
 [True, True]
 
 .. index:: random
-   :refguide: random;distributions, random;gauss""")
+   :refguide: random;distributions, random;gauss""",
+    )
 
 
 def test_yield_str():
-    line_by_line_compare(str(doc_yields),
-"""Test generator
+    line_by_line_compare(
+        str(doc_yields),
+        """Test generator
 
 Yields
 ------
@@ -493,12 +513,14 @@ b : int
     The number of bananas.
 int
     The number of unknowns.
-""")
+""",
+    )
 
 
 def test_receives_str():
-    line_by_line_compare(str(doc_sent),
-"""Test generator
+    line_by_line_compare(
+        str(doc_sent),
+        """Test generator
 
 Yields
 ------
@@ -511,29 +533,44 @@ b : int
     The number of bananas.
 c : int
     The number of oranges.
-""")
+""",
+    )
 
 
 def test_no_index_in_str():
-    assert "index" not in str(NumpyDocString("""Test idx
+    assert "index" not in str(
+        NumpyDocString(
+            """Test idx
 
-    """))
+    """
+        )
+    )
 
-    assert "index" in str(NumpyDocString("""Test idx
+    assert "index" in str(
+        NumpyDocString(
+            """Test idx
 
     .. index :: random
-    """))
+    """
+        )
+    )
 
-    assert "index" in str(NumpyDocString("""Test idx
+    assert "index" in str(
+        NumpyDocString(
+            """Test idx
 
     .. index ::
         foo
-    """))
+    """
+        )
+    )
+
 
 def test_sphinx_str():
     sphinx_doc = SphinxDocString(doc_txt)
-    line_by_line_compare(str(sphinx_doc),
-"""
+    line_by_line_compare(
+        str(sphinx_doc),
+        """
 .. index:: random
    single: random;distributions, random;gauss
 
@@ -560,6 +597,9 @@ of the one-dimensional normal distribution to higher dimensions.
         generated, and packed in an m-by-n-by-k arrangement.  Because
         each sample is N-dimensional, the output shape is (m,n,k,N).
 
+    **dtype** : data type object, optional (default : float)
+        The type and size of the data to be returned.
+
 :Returns:
 
     **out** : ndarray
@@ -602,6 +642,8 @@ of the one-dimensional normal distribution to higher dimensions.
         ..
     :obj:`otherfunc`
         relationship
+    :py:meth:`spyder.widgets.mixins.GetHelpMixin.show_object_info`
+        ..
 
 .. rubric:: Notes
 
@@ -647,13 +689,15 @@ standard deviation:
 
 >>> print(list((x[0, 0, :] - mean) < 0.6))
 [True, True]
-""")
+""",
+    )
 
 
 def test_sphinx_yields_str():
     sphinx_doc = SphinxDocString(doc_yields_txt)
-    line_by_line_compare(str(sphinx_doc),
-"""Test generator
+    line_by_line_compare(
+        str(sphinx_doc),
+        """Test generator
 
 :Yields:
 
@@ -665,10 +709,12 @@ def test_sphinx_yields_str():
 
     int
         The number of unknowns.
-""")
+""",
+    )
 
 
-doc2 = NumpyDocString("""
+doc2 = NumpyDocString(
+    """
     Returns array of indices of the maximum values of along the given axis.
 
     Parameters
@@ -677,39 +723,43 @@ doc2 = NumpyDocString("""
         Array to look in.
     axis : {None, integer}
         If None, the index is into the flattened array, otherwise along
-        the specified axis""")
+        the specified axis"""
+)
 
 
 def test_parameters_without_extended_description():
-    assert len(doc2['Parameters']) == 2
+    assert len(doc2["Parameters"]) == 2
 
 
-doc3 = NumpyDocString("""
+doc3 = NumpyDocString(
+    """
     my_signature(*params, **kwds)
 
     Return this and that.
-    """)
+    """
+)
 
 
 def test_escape_stars():
-    signature = str(doc3).split('\n')[0]
-    assert signature == r'my_signature(\*params, \*\*kwds)'
+    signature = str(doc3).split("\n")[0]
+    assert signature == r"my_signature(\*params, \*\*kwds)"
 
     def my_func(a, b, **kwargs):
         pass
 
     fdoc = FunctionDoc(func=my_func)
-    assert fdoc['Signature'] == ''
+    assert fdoc["Signature"] == ""
 
 
 doc4 = NumpyDocString(
     """a.conj()
 
-    Return an array with all complex-valued elements conjugated.""")
+    Return an array with all complex-valued elements conjugated."""
+)
 
 
 def test_empty_extended_summary():
-    assert doc4['Extended Summary'] == []
+    assert doc4["Extended Summary"] == []
 
 
 doc5 = NumpyDocString(
@@ -725,29 +775,37 @@ doc5 = NumpyDocString(
     -----
     SomeWarning
         If needed
-    """)
+    """
+)
 
 
 def test_raises():
-    assert len(doc5['Raises']) == 1
-    param = doc5['Raises'][0]
-    assert param.name == ''
-    assert param.type == 'LinAlgException'
-    assert param.desc == ['If array is singular.']
+    assert len(doc5["Raises"]) == 1
+    param = doc5["Raises"][0]
+    assert param.name == ""
+    assert param.type == "LinAlgException"
+    assert param.desc == ["If array is singular."]
 
 
 def test_warns():
-    assert len(doc5['Warns']) == 1
-    param = doc5['Warns'][0]
-    assert param.name == ''
-    assert param.type == 'SomeWarning'
-    assert param.desc == ['If needed']
+    assert len(doc5["Warns"]) == 1
+    param = doc5["Warns"][0]
+    assert param.name == ""
+    assert param.type == "SomeWarning"
+    assert param.desc == ["If needed"]
 
 
-def test_see_also():
+# see numpydoc/numpydoc #281
+# we want to correctly parse "See Also" both in docstrings both like
+# """foo
+# and
+# """
+# foo
+@pytest.mark.parametrize("prefix", ["", "\n    "])
+def test_see_also(prefix):
     doc6 = NumpyDocString(
-    """
-    z(x,theta)
+        prefix
+        + """z(x,theta)
 
     See Also
     --------
@@ -763,58 +821,68 @@ def test_see_also():
     :obj:`~baz.obj_r`
     :class:`class_j`: fubar
         foobar
-    """)
+    """
+    )
 
-    assert len(doc6['See Also']) == 10
-    for funcs, desc in doc6['See Also']:
+    assert len(doc6["See Also"]) == 10
+    for funcs, desc in doc6["See Also"]:
         for func, role in funcs:
-            if func in ('func_a', 'func_b', 'func_c', 'func_f',
-                        'func_g', 'func_h', 'func_j', 'func_k', 'baz.obj_q',
-                        'func_f1', 'func_g1', 'func_h1', 'func_j1',
-                        '~baz.obj_r'):
+            if func in (
+                "func_a",
+                "func_b",
+                "func_c",
+                "func_f",
+                "func_g",
+                "func_h",
+                "func_j",
+                "func_k",
+                "baz.obj_q",
+                "func_f1",
+                "func_g1",
+                "func_h1",
+                "func_j1",
+                "~baz.obj_r",
+            ):
                 assert not desc, str([func, desc])
-            elif func in ('func_f2', 'func_g2', 'func_h2', 'func_j2'):
+            elif func in ("func_f2", "func_g2", "func_h2", "func_j2"):
                 assert desc, str([func, desc])
             else:
                 assert desc, str([func, desc])
 
-            if func == 'func_h':
-                assert role == 'meth'
-            elif func == 'baz.obj_q' or func == '~baz.obj_r':
-                assert role == 'obj'
-            elif func == 'class_j':
-                assert role == 'class'
-            elif func in ['func_h1', 'func_h2']:
-                assert role == 'meth'
+            if func == "func_h":
+                assert role == "meth"
+            elif func == "baz.obj_q" or func == "~baz.obj_r":
+                assert role == "obj"
+            elif func == "class_j":
+                assert role == "class"
+            elif func in ["func_h1", "func_h2"]:
+                assert role == "meth"
             else:
                 assert role is None, str([func, role])
 
-            if func == 'func_d':
-                assert desc == ['some equivalent func']
-            elif func == 'foo.func_e':
-                assert desc == ['some other func over', 'multiple lines']
-            elif func == 'class_j':
-                assert desc == ['fubar', 'foobar']
-            elif func in ['func_f2', 'func_g2', 'func_h2', 'func_j2']:
-                assert desc == ['description of multiple'], str([desc, ['description of multiple']])
+            if func == "func_d":
+                assert desc == ["some equivalent func"]
+            elif func == "foo.func_e":
+                assert desc == ["some other func over", "multiple lines"]
+            elif func == "class_j":
+                assert desc == ["fubar", "foobar"]
+            elif func in ["func_f2", "func_g2", "func_h2", "func_j2"]:
+                assert desc == ["description of multiple"], str(
+                    [desc, ["description of multiple"]]
+                )
 
 
 def test_see_also_parse_error():
-    text = (
-    """
+    text = """
     z(x,theta)
 
     See Also
     --------
     :func:`~foo`
-    """)
-    with assert_raises(ParseError) as err:
+    """
+    with pytest.raises(ValueError, match="See Also entry ':func:`~foo`'"):
         NumpyDocString(text)
 
-    s1 = str(r":func:`~foo` is not a item name in '\n    z(x,theta)\n\n    See Also\n    --------\n    :func:`~foo`\n    '")
-    s2 = str(err.value)
-    assert s1 == s2
-
 
 def test_see_also_print():
     class Dummy:
@@ -826,17 +894,21 @@ def test_see_also_print():
                  goes here
         func_d
         """
+
         pass
 
-    s = str(FunctionDoc(Dummy, role='func'))
-    assert(':func:`func_a`, :func:`func_b`' in s)
-    assert('    some relationship' in s)
-    assert(':func:`func_d`' in s)
+    s = str(FunctionDoc(Dummy, role="func"))
+    assert ":func:`func_a`, :func:`func_b`" in s
+    assert "    some relationship" in s
+    assert ":func:`func_d`" in s
 
 
 def test_see_also_trailing_comma_warning():
-    warnings.filterwarnings('error')
-    with assert_warns(Warning, match='Unexpected comma or period after function list at index 43 of line .*'):
+    warnings.filterwarnings("error")
+    with assert_warns(
+        Warning,
+        match="Unexpected comma or period after function list at index 43 of line .*",
+    ):
         NumpyDocString(
             """
             z(x,theta)
@@ -846,7 +918,8 @@ def test_see_also_trailing_comma_warning():
             func_f2, func_g2, :meth:`func_h2`, func_j2, : description of multiple
             :class:`class_j`: fubar
                 foobar
-            """)
+            """
+        )
 
 
 def test_unknown_section():
@@ -865,42 +938,65 @@ This should be ignored and warned about
         ----
         This class has a nope section.
         """
+
         pass
 
-    with warnings.catch_warnings(record=True) as w:
-        warnings.filterwarnings('always', '', UserWarning)
+    with pytest.warns(UserWarning, match="Unknown section Mope") as record:
         NumpyDocString(doc_text)
-        assert len(w) == 1
-        assert "Unknown section Mope" == str(w[0].message)
+    assert len(record) == 1
 
-    with warnings.catch_warnings(record=True) as w:
-        warnings.filterwarnings('always', '', UserWarning)
+    # SphinxClassDoc has _obj.__name__ == "BadSection". Test that this is
+    # included in the message
+    msg_match = "Unknown section Nope in the docstring of BadSection"
+    with pytest.warns(UserWarning, match=msg_match) as record:
         SphinxClassDoc(BadSection)
-        assert len(w) == 1
-        assert('test_docscrape.test_unknown_section.<locals>.BadSection'
-               in str(w[0].message)
-               or 'test_docscrape.BadSection' in str(w[0].message))
+    assert len(record) == 1
 
 
-doc7 = NumpyDocString("""
+doc7 = NumpyDocString(
+    """
 
         Doc starts on second line.
 
-        """)
+        """
+)
 
 
 def test_empty_first_line():
-    assert doc7['Summary'][0].startswith('Doc starts')
+    assert doc7["Summary"][0].startswith("Doc starts")
+
+
+doc8 = NumpyDocString(
+    """
+
+        Parameters with colon and no types:
+
+        Parameters
+        ----------
+
+        data :
+            some stuff, technically invalid
+        """
+)
+
+
+def test_trailing_colon():
+    assert doc8["Parameters"][0].name == "data"
 
 
 def test_no_summary():
-    str(SphinxDocString("""
+    str(
+        SphinxDocString(
+            """
     Parameters
-    ----------"""))
+    ----------"""
+        )
+    )
 
 
 def test_unicode():
-    doc = SphinxDocString("""
+    doc = SphinxDocString(
+        """
     öäöäöäöäöåååå
 
     öäöäöäööäååå
@@ -915,33 +1011,41 @@ def test_unicode():
     ååå : ööö
         äää
 
-    """)
-    assert isinstance(doc['Summary'][0], str)
-    assert doc['Summary'][0] == 'öäöäöäöäöåååå'
+    """
+    )
+    assert isinstance(doc["Summary"][0], str)
+    assert doc["Summary"][0] == "öäöäöäöäöåååå"
 
 
 def test_plot_examples():
     cfg = dict(use_plots=True)
 
-    doc = SphinxDocString("""
+    doc = SphinxDocString(
+        """
     Examples
     --------
     >>> import matplotlib.pyplot as plt
     >>> plt.plot([1,2,3],[4,5,6])
     >>> plt.show()
-    """, config=cfg)
-    assert 'plot::' in str(doc), str(doc)
+    """,
+        config=cfg,
+    )
+    assert "plot::" in str(doc), str(doc)
 
-    doc = SphinxDocString("""
+    doc = SphinxDocString(
+        """
     Examples
     --------
     >>> from matplotlib import pyplot as plt
     >>> plt.plot([1,2,3],[4,5,6])
     >>> plt.show()
-    """, config=cfg)
-    assert 'plot::' in str(doc), str(doc)
+    """,
+        config=cfg,
+    )
+    assert "plot::" in str(doc), str(doc)
 
-    doc = SphinxDocString("""
+    doc = SphinxDocString(
+        """
     Examples
     --------
     .. plot::
@@ -949,13 +1053,16 @@ def test_plot_examples():
        import matplotlib.pyplot as plt
        plt.plot([1,2,3],[4,5,6])
        plt.show()
-    """, config=cfg)
-    assert str(doc).count('plot::') == 1, str(doc)
+    """,
+        config=cfg,
+    )
+    assert str(doc).count("plot::") == 1, str(doc)
 
 
 def test_use_blockquotes():
     cfg = dict(use_blockquotes=True)
-    doc = SphinxDocString("""
+    doc = SphinxDocString(
+        """
     Parameters
     ----------
     abc : def
@@ -969,8 +1076,12 @@ def test_use_blockquotes():
         GHI
     JKL
         MNO
-    """, config=cfg)
-    line_by_line_compare(str(doc), '''
+    """,
+        config=cfg,
+    )
+    line_by_line_compare(
+        str(doc),
+        """
     :Parameters:
 
         **abc** : def
@@ -990,22 +1101,25 @@ def test_use_blockquotes():
         JKL
 
             MNO
-    ''')
+    """,
+    )
 
 
 def test_class_members():
-
     class Dummy:
         """
         Dummy class.
 
         """
+
         def spam(self, a, b):
             """Spam\n\nSpam spam."""
             pass
+
         def ham(self, c, d):
             """Cheese\n\nNo cheese."""
             pass
+
         @property
         def spammity(self):
             """Spammity index"""
@@ -1013,32 +1127,34 @@ def test_class_members():
 
         class Ignorable:
             """local class, to be ignored"""
+
             pass
 
     for cls in (ClassDoc, SphinxClassDoc):
         doc = cls(Dummy, config=dict(show_class_members=False))
-        assert 'Methods' not in str(doc), (cls, str(doc))
-        assert 'spam' not in str(doc), (cls, str(doc))
-        assert 'ham' not in str(doc), (cls, str(doc))
-        assert 'spammity' not in str(doc), (cls, str(doc))
-        assert 'Spammity index' not in str(doc), (cls, str(doc))
+        assert "Methods" not in str(doc), (cls, str(doc))
+        assert "spam" not in str(doc), (cls, str(doc))
+        assert "ham" not in str(doc), (cls, str(doc))
+        assert "spammity" not in str(doc), (cls, str(doc))
+        assert "Spammity index" not in str(doc), (cls, str(doc))
 
         doc = cls(Dummy, config=dict(show_class_members=True))
-        assert 'Methods' in str(doc), (cls, str(doc))
-        assert 'spam' in str(doc), (cls, str(doc))
-        assert 'ham' in str(doc), (cls, str(doc))
-        assert 'spammity' in str(doc), (cls, str(doc))
+        assert "Methods" in str(doc), (cls, str(doc))
+        assert "spam" in str(doc), (cls, str(doc))
+        assert "ham" in str(doc), (cls, str(doc))
+        assert "spammity" in str(doc), (cls, str(doc))
 
         if cls is SphinxClassDoc:
-            assert '.. autosummary::' in str(doc), str(doc)
+            assert ".. autosummary::" in str(doc), str(doc)
         else:
-            assert 'Spammity index' in str(doc), str(doc)
+            assert "Spammity index" in str(doc), str(doc)
 
     class SubDummy(Dummy):
         """
         Subclass of Dummy class.
 
         """
+
         def ham(self, c, d):
             """Cheese\n\nNo cheese.\nOverloaded Dummy.ham"""
             pass
@@ -1048,31 +1164,35 @@ def test_class_members():
             pass
 
     for cls in (ClassDoc, SphinxClassDoc):
-        doc = cls(SubDummy, config=dict(show_class_members=True,
-                                        show_inherited_class_members=False))
-        assert 'Methods' in str(doc), (cls, str(doc))
-        assert 'spam' not in str(doc), (cls, str(doc))
-        assert 'ham' in str(doc), (cls, str(doc))
-        assert 'bar' in str(doc), (cls, str(doc))
-        assert 'spammity' not in str(doc), (cls, str(doc))
+        doc = cls(
+            SubDummy,
+            config=dict(show_class_members=True, show_inherited_class_members=False),
+        )
+        assert "Methods" in str(doc), (cls, str(doc))
+        assert "spam" not in str(doc), (cls, str(doc))
+        assert "ham" in str(doc), (cls, str(doc))
+        assert "bar" in str(doc), (cls, str(doc))
+        assert "spammity" not in str(doc), (cls, str(doc))
 
         if cls is SphinxClassDoc:
-            assert '.. autosummary::' in str(doc), str(doc)
+            assert ".. autosummary::" in str(doc), str(doc)
         else:
-            assert 'Spammity index' not in str(doc), str(doc)
+            assert "Spammity index" not in str(doc), str(doc)
 
-        doc = cls(SubDummy, config=dict(show_class_members=True,
-                                        show_inherited_class_members=True))
-        assert 'Methods' in str(doc), (cls, str(doc))
-        assert 'spam' in str(doc), (cls, str(doc))
-        assert 'ham' in str(doc), (cls, str(doc))
-        assert 'bar' in str(doc), (cls, str(doc))
-        assert 'spammity' in str(doc), (cls, str(doc))
+        doc = cls(
+            SubDummy,
+            config=dict(show_class_members=True, show_inherited_class_members=True),
+        )
+        assert "Methods" in str(doc), (cls, str(doc))
+        assert "spam" in str(doc), (cls, str(doc))
+        assert "ham" in str(doc), (cls, str(doc))
+        assert "bar" in str(doc), (cls, str(doc))
+        assert "spammity" in str(doc), (cls, str(doc))
 
         if cls is SphinxClassDoc:
-            assert '.. autosummary::' in str(doc), str(doc)
+            assert ".. autosummary::" in str(doc), str(doc)
         else:
-            assert 'Spammity index' in str(doc), str(doc)
+            assert "Spammity index" in str(doc), str(doc)
 
 
 def test_duplicate_signature():
@@ -1081,13 +1201,14 @@ def test_duplicate_signature():
     # docstring itself.
 
     doc = NumpyDocString(
-    """
+        """
     z(x1, x2)
 
     z(a, theta)
-    """)
+    """
+    )
 
-    assert doc['Signature'].strip() == 'z(a, theta)'
+    assert doc["Signature"].strip() == "z(a, theta)"
 
 
 class_doc_txt = """
@@ -1133,8 +1254,9 @@ class_doc_txt = """
 
 def test_class_members_doc():
     doc = ClassDoc(None, class_doc_txt)
-    line_by_line_compare(str(doc),
-    """
+    line_by_line_compare(
+        str(doc),
+        """
     Foo
 
     Parameters
@@ -1172,7 +1294,8 @@ def test_class_members_doc():
     b
     c
 
-    """)
+    """,
+    )
 
 
 def test_class_members_doc_sphinx():
@@ -1211,8 +1334,9 @@ def test_class_members_doc_sphinx():
             return None
 
     doc = SphinxClassDoc(Foo, class_doc_txt)
-    line_by_line_compare(str(doc),
-    """
+    line_by_line_compare(
+        str(doc),
+        """
     Foo
 
     :Parameters:
@@ -1264,11 +1388,11 @@ def test_class_members_doc_sphinx():
     **c**
     =====  ==========
 
-    """)
+    """,
+    )
 
 
 def test_class_attributes_as_member_list():
-
     class Foo:
         """
         Class docstring.
@@ -1279,6 +1403,7 @@ def test_class_attributes_as_member_list():
             Another description that is not used.
 
         """
+
         @property
         def an_attribute(self):
             """Test attribute"""
@@ -1305,10 +1430,14 @@ def test_class_attributes_as_member_list():
 
 
 def test_templated_sections():
-    doc = SphinxClassDoc(None, class_doc_txt,
-                         config={'template': jinja2.Template('{{examples}}\n{{parameters}}')})
-    line_by_line_compare(str(doc),
-    """
+    doc = SphinxClassDoc(
+        None,
+        class_doc_txt,
+        config={"template": jinja2.Template("{{examples}}\n{{parameters}}")},
+    )
+    line_by_line_compare(
+        str(doc),
+        """
     .. rubric:: Examples
 
     For usage examples, see `ode`.
@@ -1321,14 +1450,14 @@ def test_templated_sections():
         **jac** : callable ``jac(t, y, *jac_args)``
             Bbb.
 
-    """)
+    """,
+    )
 
 
 def test_nonstandard_property():
     # test discovery of a property that does not satisfy isinstace(.., property)
 
     class SpecialProperty:
-
         def __init__(self, axis=0, doc=""):
             self.axis = axis
             self.__doc__ = doc
@@ -1353,7 +1482,8 @@ def test_nonstandard_property():
 
 def test_args_and_kwargs():
     cfg = dict()
-    doc = SphinxDocString("""
+    doc = SphinxDocString(
+        """
     Parameters
     ----------
     param1 : int
@@ -1362,8 +1492,12 @@ def test_args_and_kwargs():
         Arguments
     **kwargs : dict
         Keyword arguments
-    """, config=cfg)
-    line_by_line_compare(str(doc), r"""
+    """,
+        config=cfg,
+    )
+    line_by_line_compare(
+        str(doc),
+        r"""
 :Parameters:
 
     **param1** : int
@@ -1374,17 +1508,24 @@ def test_args_and_kwargs():
 
     **\*\*kwargs** : dict
         Keyword arguments
-    """)
+    """,
+    )
+
 
 def test_autoclass():
-    cfg=dict(show_class_members=True,
-             show_inherited_class_members=True)
-    doc = SphinxClassDoc(str, '''
+    cfg = dict(show_class_members=True, show_inherited_class_members=True)
+    doc = SphinxClassDoc(
+        str,
+        """
 A top section before
 
 .. autoclass:: str
-    ''', config=cfg)
-    line_by_line_compare(str(doc), r'''
+    """,
+        config=cfg,
+    )
+    line_by_line_compare(
+        str(doc),
+        r"""
 A top section before
 
 .. autoclass:: str
@@ -1392,7 +1533,9 @@ A top section before
 .. rubric:: Methods
 
 
-    ''', 5)
+    """,
+        5,
+    )
 
 
 xref_doc_txt = """
@@ -1455,35 +1598,64 @@ Test xref in Parameters, Other Parameters and Returns
 
 def test_xref():
     xref_aliases = {
-        'sequence': ':obj:`python:sequence`',
+        "sequence": ":obj:`python:sequence`",
     }
 
-    class Config():
+    class Config:
         def __init__(self, a, b):
             self.numpydoc_xref_aliases = a
             self.numpydoc_xref_aliases_complete = b
+            # numpydoc.update_config fails if this config option not present
+            self.numpydoc_validation_checks = set()
+            self.numpydoc_validation_exclude = set()
 
     xref_aliases_complete = deepcopy(DEFAULT_LINKS)
     for key in xref_aliases:
         xref_aliases_complete[key] = xref_aliases[key]
     config = Config(xref_aliases, xref_aliases_complete)
-    app = namedtuple('config', 'config')(config)
+    app = namedtuple("config", "config")(config)
     update_config(app)
 
-    xref_ignore = {'of', 'default', 'optional'}
+    xref_ignore = {"of", "default", "optional"}
 
     doc = SphinxDocString(
         xref_doc_txt,
         config=dict(
             xref_param_type=True,
             xref_aliases=xref_aliases_complete,
-            xref_ignore=xref_ignore
-        )
+            xref_ignore=xref_ignore,
+        ),
     )
 
     line_by_line_compare(str(doc), xref_doc_txt_expected)
 
 
+def test__error_location_no_name_attr():
+    """
+    Ensure that NumpyDocString._error_location doesn't fail when self._obj
+    does not have a __name__ attr.
+
+    See gh-362
+    """
+    from collections.abc import Callable
+
+    # Create a Callable that doesn't have a __name__ attribute
+    class Foo:
+        def __call__(self):
+            pass
+
+    foo = Foo()  # foo is a Callable, but no a function instance
+    assert isinstance(foo, Callable)
+
+    # Create an NumpyDocString instance to call the _error_location method
+    nds = get_doc_object(foo)
+
+    msg = "Potentially wrong underline length.*Foo.*"
+    with pytest.raises(ValueError, match=msg):
+        nds._error_location(msg=msg)
+
+
 if __name__ == "__main__":
     import pytest
+
     pytest.main()
